#!/usr/bin/env node
"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || (function () {
    var ownKeys = function(o) {
        ownKeys = Object.getOwnPropertyNames || function (o) {
            var ar = [];
            for (var k in o) if (Object.prototype.hasOwnProperty.call(o, k)) ar[ar.length] = k;
            return ar;
        };
        return ownKeys(o);
    };
    return function (mod) {
        if (mod && mod.__esModule) return mod;
        var result = {};
        if (mod != null) for (var k = ownKeys(mod), i = 0; i < k.length; i++) if (k[i] !== "default") __createBinding(result, mod, k[i]);
        __setModuleDefault(result, mod);
        return result;
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
const child_process_1 = require("child_process");
const commander_1 = require("commander");
const admin = __importStar(require("firebase-admin"));
const tools = __importStar(require("firebase-tools"));
const firestore_export_import_1 = require("firestore-export-import");
const fs_1 = require("fs");
const util = __importStar(require("util"));
const program = new commander_1.Command();
const execPromise = util.promisify(child_process_1.exec);
admin.initializeApp({ credential: admin.credential.applicationDefault() });
const authInstance = admin.auth();
const firestoreInstance = admin.firestore();
const FIRESTORE_DATA_DIR = 'data/firestore';
const FIRESTORE_DATA_PATH = `${FIRESTORE_DATA_DIR}/collections.json`;
const AUTH_DATA_DIR = 'data/auth';
const AUTH_DATA_PATH = `${AUTH_DATA_DIR}/users.json`;
const STORAGE_DATA_DIR = 'data/storage';
const fileExists = async (path) => {
    return await fs_1.promises.access(path)
        .then(() => true)
        .catch(() => false);
};
const anyFileExists = async (path) => {
    const files = await fs_1.promises.readdir(path);
    return files.length > 0;
};
const directoryExists = async (path) => {
    return await fs_1.promises.stat(path)
        .then((stats) => stats.isDirectory())
        .catch(() => false);
};
const makeOrRemoveDirectory = async (path) => {
    if (await directoryExists(path)) {
        await execPromise(`rm -rf ${path}/*`);
    }
    else {
        await fs_1.promises.mkdir(path, { recursive: true });
    }
};
const backup = async (targets) => {
    if (targets.includes('firestore')) {
        await makeOrRemoveDirectory(FIRESTORE_DATA_DIR);
        const collections = await (0, firestore_export_import_1.backups)(firestoreInstance);
        await fs_1.promises.writeFile(FIRESTORE_DATA_PATH, JSON.stringify(collections, null, '  '));
    }
    if (targets.includes('auth')) {
        await makeOrRemoveDirectory(AUTH_DATA_DIR);
        await tools.auth.export(AUTH_DATA_PATH, { project: process.env.FIREBASE_PROJECT_ID });
    }
    if (targets.includes('storage')) {
        await makeOrRemoveDirectory(STORAGE_DATA_DIR);
        if ((await execPromise(`gsutil ls gs://${process.env.FIREBASE_STORAGE_BUCKET}`)).stdout) {
            // > If you experience problems with multiprocessing on MacOS, they might be related to https://bugs.python.org/issue33725.
            // > You can disable multiprocessing by editing your.boto config or by adding the following flag to your command: `-o "GSUtil:parallel_process_count=1"`.
            // > Note that multithreading is still available even if you disable multiprocessing.
            await execPromise(`gsutil -m -o "GSUtil:parallel_process_count=1" cp -r gs://${process.env.FIREBASE_STORAGE_BUCKET}/* ${STORAGE_DATA_DIR}`);
        }
    }
};
const restore = async (targets) => {
    if (targets.includes('firestore')) {
        if (await fileExists(FIRESTORE_DATA_PATH)) {
            await (0, firestore_export_import_1.restore)(firestoreInstance, FIRESTORE_DATA_PATH, {
                autoParseDates: true,
                autoParseGeos: true,
            });
        }
    }
    if (targets.includes('auth')) {
        if (await fileExists(AUTH_DATA_PATH)) {
            await tools.auth.upload(AUTH_DATA_PATH, {
                project: process.env.FIREBASE_PROJECT_ID,
                hashAlgo: 'HMAC_SHA256',
                hashKey: 'N2RheXM=',
            });
        }
    }
    if (targets.includes('storage')) {
        if (await anyFileExists(STORAGE_DATA_DIR)) {
            // > If you experience problems with multiprocessing on MacOS, they might be related to https://bugs.python.org/issue33725.
            // > You can disable multiprocessing by editing your.boto config or by adding the following flag to your command: `-o "GSUtil:parallel_process_count=1"`.
            // > Note that multithreading is still available even if you disable multiprocessing.
            await execPromise(`gsutil -m -o "GSUtil:parallel_process_count=1" cp -r ${STORAGE_DATA_DIR}/* gs://${process.env.FIREBASE_STORAGE_BUCKET}`);
        }
    }
};
const _delete = async (targets) => {
    if (targets.includes('firestore')) {
        await tools.firestore.delete({ allCollections: true });
    }
    if (targets.includes('auth')) {
        const usersResult = await authInstance.listUsers();
        const uids = usersResult.users.map((user) => user.uid);
        await authInstance.deleteUsers(uids);
    }
    if (targets.includes('storage')) {
        if ((await execPromise(`gsutil ls gs://${process.env.FIREBASE_STORAGE_BUCKET}`)).stdout) {
            await execPromise(`gsutil rm gs://${process.env.FIREBASE_STORAGE_BUCKET}/**`);
        }
    }
};
const checkEnv = () => {
    if (!process.env.FIREBASE_PROJECT_ID) {
        console.error('Missing env FIREBASE_PROJECT_ID');
        process.exit(1);
    }
    if (!process.env.FIREBASE_STORAGE_BUCKET) {
        console.error('Missing env FIREBASE_STORAGE_BUCKET');
        process.exit(1);
    }
    if (!process.env.GOOGLE_APPLICATION_CREDENTIALS) {
        console.error('Missing env GOOGLE_APPLICATION_CREDENTIALS');
        process.exit(1);
    }
};
const commaSeparatedList = (value) => {
    return value.toLowerCase().split(',');
};
program
    .version('0.1.1')
    .description('NPM package for backup, restore, delete and replace Firebase')
    .option('--only <targets>', 'only execute to specified, comma-separated targets (e.g. "firestore,storage").', commaSeparatedList);
program
    .command('backup')
    .description('backup Firestore, Authentication, Storage')
    .action(async () => {
    checkEnv();
    const options = program.opts();
    const targets = options.only || ['firestore', 'auth', 'storage'];
    await backup(targets);
});
program
    .command('restore')
    .description('restore Firestore, Authentication, Storage')
    .action(async () => {
    checkEnv();
    const options = program.opts();
    const targets = options.only || ['firestore', 'auth', 'storage'];
    await restore(targets);
});
program
    .command('delete')
    .description('delete Firestore, Authentication, Storage')
    .action(async () => {
    checkEnv();
    const options = program.opts();
    const targets = options.only || ['firestore', 'auth', 'storage'];
    await _delete(targets);
});
program
    .command('replace')
    .description('delete and restore')
    .action(async () => {
    checkEnv();
    const options = program.opts();
    const targets = options.only || ['firestore', 'auth', 'storage'];
    await _delete(targets);
    await restore(targets);
});
program.parse();
//# sourceMappingURL=index.js.map